########################################################
# Landsat Data
########################################################
# Version: June 25, 2018


## Required functions in fitting Landsat Data

##############################################################################
# Generate unifom (copula) data
##############################################################################

SA.rCopula <- function(N, family, par){
  
  cop.data <- NULL
  
  for(i in 1:length(N)){
    
    if(N[i] == 1) {  ## If dim=1, it is calculated using uniform distribution[0,1]
      
      U <- runif(1)
      
    }else {          ## If dim >=2, it is calculated using Clayton Copula
      
      if (family == c("normal")){ ## Gaussian Copula
        
        mycop <- copula::normalCopula(par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
      
      if (family == c("t")){ ## t Copula
        
        mycop <- copula::tCopula(par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
      
      if(family == "clayton" | family == "gumbel" | family == "frank" | family == "joe") { 
        
        mycop <- copula::archmCopula(family = family, par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
    }
    
    AreaID <- as.matrix(seq(from = i, to = i, length.out = N[i])) ## Constructing a new variable which displays the areaID, thus to distinguish between each area
    U.area <- cbind(U, AreaID)
    cop.data <- rbind(cop.data, U.area)
  }
  
  return(list(U = cop.data[,1], AreaID = cop.data[,2]))
  
}



##############################################################################
# Copula log-likehood function for estimation of the copula parameter
##############################################################################
SA.LLIK <- function(u, areaID, family, par){
  
  n.area = as.numeric(table(areaID))
  cop.dens = rep(NA,length(n.area))
  
  for(i in 1:length(n.area)){
    
    if(n.area[i]>1){
      u.area <- subset(u, areaID == i)
      
      if(family == "normal"){
        mycop <- copula::normalCopula(par=par , dim= n.area[i])
      }
      
      if(family == "t"){
        mycop <- copula::tCopula(par=par , dim= n.area[i])
      }
      
      if(family == "clayton" | family == "gumbel" | family == "frank") { 
        mycop <- copula::archmCopula(family = family, par=par , dim= n.area[i])
      }
      
      cop.dens[i] <- copula::dCopula(u.area, mycop, log = TRUE)
    }
    else{cop.dens[i] <- 0}
  }
  
  return(sum(cop.dens[is.finite(cop.dens)], rm.na=T))
}




##############################################################################
# Likelihood-based Estimation of the Copula Parameter
##############################################################################

SA.CopEst <- function(u, areaID, family){
  
  obj.function= function(par){-SA.LLIK(u, areaID, family=family, par=par)}
  
  if(family == "normal" | family == "t"){
    res<-optimize(obj.function, lower=0, upper=1)
  }
  
  if(family == "gumbel"){
    res<-optimize(obj.function, lower=1, upper=100)
  }
  
  if(family == "frank" | family == "clayton"){
    res<-optimize(obj.function, lower=0, upper=10)
  }
  
  return(list(par = res$minimum, llik = -res$objective ))
}



##############################################################################
# Converting Error to Copula Data using a Parametric Model
##############################################################################

SA.Error2U <- function(error, mar.dist, mar.par){
  
  if(mar.dist == "norm"){
    if(length(mar.par)!=2) stop("Provide mean and variance")
    u <- pnorm(error, mean = mar.par[1], sd = sqrt(mar.par[2])) ## Calculating CDF
  }
  
  if(mar.dist == "snorm"){
    if(length(mar.par)!=3) stop("Provide three parameters")
    u <- psn(error, xi=mar.par[3], omega = mar.par[1], alpha = mar.par[2]) ## Calculating CDF
  }
  
  return(u) 
}



##############################################################################
# Converting Copula Data to Error using a Parametric Model
##############################################################################

SA.U2Error <- function(u, mar.dist, mar.par){
  
  if(mar.dist == "norm"){
    if(length(mar.par)!=2) stop("Provide mean and variance")
    error <- qnorm(u, mean = mar.par[1], sd = sqrt(mar.par[2])) ## Calculating Inverse CDF
  }
  
  
  if(mar.dist == "snorm"){
    if(length(mar.par)!=3) stop("Provide three parameters")
    error <- qsn(u, xi=mar.par[3], omega = mar.par[1], alpha = mar.par[2]) ## Calculating Inverse CDF
  }
  
  return(error) 
}





#############################################################################
## Weight Function (w1i)
##############################################################################

SA.weight <- function(u,  u.i,  family, par){
  
  n <- length(u)       ## CDF of all residuals
  nSAi <- length(u.i)  ## CDF of selected small area
  
  Fe_matrix <- matrix(u.i, nrow = n, ncol = nSAi, byrow = TRUE)
  
  if(nSAi != 1){
    if(family == "normal"){
      cop.num <- copula::normalCopula( par, dim = (nSAi+1))
      cop.den <- copula::normalCopula( par, dim = nSAi)
    }
    
    if(family == "t"){
      cop.num <- copula::tCopula( par, dim = (nSAi+1))
      cop.den <- copula::tCopula( par, dim = nSAi)
    }
    
    if(family == "clayton" | family == "gumbel" | family == "frank") { 
      cop.num <- copula::archmCopula(family = family, par, dim = (nSAi+1))
      cop.den <- copula::archmCopula(family = family, par, dim = nSAi)
    }
    
    w1.i_num <- copula::dCopula(cbind(u, Fe_matrix), cop.num)
    w1.i_den <- copula::dCopula(u.i, cop.den)
    w1.i <- w1.i_num/w1.i_den
  }
  
  else {
    if(family == "normal"){
      cop.num <- copula::normalCopula( par, dim = (nSAi+1))
    }
    
    if(family == "t"){
      cop.num <- copula::tCopula( par, dim = (nSAi+1))
    }
    
    if(family == "clayton" | family == "gumbel" | family == "frank") { 
      cop.num <- copula::archmCopula(family = family, par, dim = (nSAi+1))
    }
    
    w1.i_num <- copula::dCopula(cbind(u, Fe_matrix), cop.num)
    w1.i <- w1.i_num
  }
  
  return(w1.i)
}




#############################################################################
## Empirical predictor function for unsampled error terms
##############################################################################

SA.pred.error <- function(error, u, areaID, family, par){
  
  m <- length(unique(areaID))
  num.areaid <- as.numeric(table(areaID))
  
  emp.pred.areai <- NULL  
  for(i in 1:m){
    if(num.areaid[i] == 1) {
      u.i <- subset(u, areaID == i)
      w1.i <- SA.weight(u=u, u.i= u.i,  family = family, par=par)
      emp.pred.areai[i] =  sum(error * w1.i) / sum(w1.i)
    }
    else {   
      u.i <- subset(u, areaID == i)
      w1.i <- SA.weight(u=u, u.i= u.i,  family = family, par=par)
      emp.pred.areai[i] <- sum(error * w1.i) / sum(w1.i)
    }
  }
  return(emp.pred.areai)
}



#############################################################################
## Empirical BUP / BUP (Depending upon values)
##############################################################################

SA.pred.mean <- function(N, n, e.hat, e.bar, beta, X1.bar, X2.bar){
  
  m <- length(X1.bar)
#  if(length(N) != m | length(n) != m) Stop("Number of small areas does not match for sample and/or population.")
  
  Yi <- NULL
  
  for(i in 1:m){
    l1 <- (beta[1]+(X1.bar[i]*beta[2]) +X2.bar[i]*beta[3] )
    l2 <- ((n[i]/N[i])*e.bar[i])
    l3 <- ((N[i]-n[i])/N[i])
    l4 <- e.hat[i]
    Yi[i] <-  l1 + l2 + (l3*l4)
  }
  
  return(Yi)
}

