########################################################
# Landsat Data
########################################################
# Version: August 19, 2019


## Data Application

## Caution: Run all required functions to proceed further as defined in Landsat_Functions.R

## Required packages
library(VineCopula)
library(copula)
library(dplyr)
library(sn)
library(rsae)



## 1. Read and load the Landsat Data set
help("landsat")
data("landsat")



## 2. Select all covariated required for fitting
landsat[c("HACorn", "PixelsCorn", "PixelsSoybeans", "CountyName", "outlier", "SegmentsInCounty", "SegementID")]



## 3. Remove the outlier from data set
landsat_out <- landsat[c(-33),]



## 4. Some prelminary information 
family = "frank"
method = "par" ## Define method: "par"=parametric and "spar"=semi-parametric
mar.dist = "snorm"  ## Define marginal distribution: "norm"=normal and "snorm"=skewed normal 
B = 1000 ## Number of bootstrap runs



## 5. A list of all information from data
areaID <- group_indices(landsat_out, CountyName)
CountyName <- landsat_out$CountyName
attach(landsat_out)

m = length(unique(areaID))
n = as.numeric(table(areaID))
N = unique(SegmentsInCounty)

X1.bar.aux <- unique(MeanPixelsCorn)
X2.bar.aux <- unique(MeanPixelsSoybeans)
Y.bar = aggregate(HACorn, by = list(CountyName), FUN = "mean")$x



## Function to fit Landsat dataset
CopSApred <- function(m,n, N, Y, X1, X2, X1.bar.aux, X2.bar.aux, Y.bar, areaID, CountyName, family, method, mar.dist, B){
  
    fit.margin <- lm(Y ~ X1 + X2) 
    beta <- as.numeric(fit.margin$coefficients)
    resids <- as.numeric(resid(fit.margin))
    residbar <- aggregate(resids, by = list(CountyName), FUN = "mean")$x  # Mean of residuals for each area
    
    
    ## Estimate Copula Parameter:
    
    ### Parametric Method
    if(method == "par"){
      if(mar.dist == "snorm"){
        x.sn <-  matrix(1, nrow = length(resids), ncol = 1)
        sn.est <-  sn::selm.fit(x.sn, resids, family = "SN", selm.control = "MLE")
        mar.par <- as.numeric(sn.est$param$dp[c(2,3,1)])
        
        u = SA.Error2U(resids, mar.dist = c("snorm"), mar.par)
        cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
        par.hat <- cop.est$par
      }
      
      if(mar.dist == "norm"){
        mar.par <- mar.par <- c(mean(resids), var(resids))
        
        u = SA.Error2U(resids, mar.dist = c("norm"), mar.par)
        cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
        par.hat <- cop.est$par
      }
    }
    
    
    ### Semi-parametric Method
    if(method == "spar"){
      est.method = "mle" ## Only the mle version is proposed
      if(est.method == "mle"){
        #CDF transformation of residuals under unknown marginal distribution
        u <- copula::pobs(resids)
        cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
        par.hat = cop.est$par
      }
    }
    
    
    error <-  HACorn -(PixelsSoybeans + PixelsCorn)  ## Sample errors
    error.bar <- aggregate(error, by = list(areaID), FUN = "mean")$x  ## Sample mean errors
    emp.pred <- SA.pred.error(resids, u, areaID, family, par.hat)  ## empirical predictor
    
    
    ## Small area mean predicted value
    Yi <- SA.pred.mean(N, n, e.hat=emp.pred, e.bar=error.bar, beta=beta, X1.bar=X1.bar.aux, X2.bar=X2.bar.aux)

    
    
        
    
    ## Bootstrap Method
    set.seed(1)
    mse.b = matrix(NA, nrow = B, ncol = m)
    err.b <- matrix(NA, nrow = B, ncol = m)
    
    for (b in 1:B) {
       
      U <- SA.rCopula(n, family=family, par=par.hat)$U
      
      
      ## Parametric Methos
      if(method == "par"){
        Error = SA.U2Error(U, mar.dist = mar.dist, mar.par=mar.par)
        y.boot <- (beta[1]+(X1*beta[2]) + (X2*beta[3])) + Error
        
        fit.margin.b <- lm(y.boot ~  X1 + X2)
        beta.b <- as.numeric(fit.margin.b$coefficients)
        resids.b <- as.numeric(resid(fit.margin.b))
        residbar.b <- aggregate(resids.b, by = list(CountyName), FUN = "mean")$x  # Mean of residuals for each area
        err.b[b,] <-   residbar.b^2
        
      
        if(mar.dist == "snorm"){
          x.sn <-  matrix(1, nrow = length(resids.b), ncol = 1)
          sn.est <-  sn::selm.fit(x.sn, resids.b, family = "SN", selm.control = "MLE")
          mar.par.b <- as.numeric(sn.est$param$dp[c(2,3,1)])
        
          u.b <- SA.Error2U(error = resids.b, mar.dist=mar.dist, mar.par=mar.par.b)
          cop.est.b <- SA.CopEst(u=u.b, areaID=areaID, family=family)
          par.hat.b = cop.est.b$par
        }
        
        if(mar.dist == "norm"){
          mar.par <- mar.par <- c(mean(resids), var(resids))
          
          u = SA.Error2U(resids, mar.dist = c("norm"), mar.par)
          cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
          par.hat <- cop.est$par
        }
      }
      
      
      
      ## Semi-parametric Method
      if(method == "spar"){
        # Quantile mapping in the range of observed error (residuals) based on sample:
        Error = rep(NA,length(U))
        sort.e = sort(resids)
        sort.v = copula::pobs(sort.e)
        
        for(j in 1:length(U)){
          if(U[j]>=max(sort.v)){ Error[j]=max(resids)
          }else if( U[j] <= min(sort.v)){ Error[j]=min(resids)
          }else{
            loc = findInterval(U[j], sort.v)
            v.int = sort.v[c(loc, loc+1)]
            e.int = sort.e[c(loc, loc+1)]
            Error[j] = approx(v.int, e.int, xout= U[j], yleft= min(resids), yright=max(resids))$y  
          }}
        
        y.boot <- (beta[1]+(PixelsCorn*beta[2]) + (PixelsSoybeans*beta[3])) + Error
        
        fit.margin.b <- lm(y.boot ~  PixelsCorn + PixelsSoybeans )
        beta.b <- as.numeric(fit.margin.b$coefficients)
        resids.b <- as.numeric(resid(fit.margin.b))
        residbar.b <- aggregate(resids.b, by = list(CountyName), FUN = "mean")$x  # Mean of residuals for each area
        err.b[b,] <-   residbar.b^2
        
        # CDF transformation of residuals under unknown marginal distribution
        u.b <- copula::pobs(resids.b)
          cop.est.b <- SA.CopEst(u=u.b, areaID=areaID, family=family)
          par.hat.b = cop.est.b$par
      }
      
      
      emp.pred.b <- SA.pred.error(resids.b, u.b, areaID, family, par.hat.b) ## empirical predicted value in bootstrap
      error.bar.b <- aggregate(Error, by = list(CountyName), FUN="mean")$x  ## sampled mean error in bootstrap 
      
      
      ## Predicted small area means
      Yi.b <- SA.pred.mean(N, n, e.hat=emp.pred.b, e.bar=error.bar.b, beta=beta.b, X1.bar=X1.bar.aux, X2.bar=X2.bar.aux)
      Y.bar.boot = beta[1] + beta[2]*X1.bar.aux + beta[3]*X2.bar.aux + error.bar.b
      
      
      ## Restore the value of estimated Mean squared error calculated during bootstrap procedure
      mse.b[b,] <-((Yi.b - Y.bar.boot)^2)
    }
    
    mse.b <- na.omit(mse.b)
    Rmspe <- round(sqrt(apply(mse.b,2,mean)),1)
    Yi <- round(Yi,1)
    est.par <- sn.est$param$dp
    resid <- resids
    return(list(Yi = Yi, Rmspe = Rmspe,est.par=est.par, resid=resids, margins = fit.margin$coefficients, fit.par = sn.est$param$dp))
}






## Result for section 5: Data Application
Landsat.fit <-CopSApred(m,n, N, Y=HACorn, X1 = PixelsCorn, X2 = PixelsSoybeans, X1.bar.aux, X2.bar.aux, Y.bar, areaID, CountyName, family, method, mar.dist, B)

## EBUP
Landsat.fit$Yi

## Root mean squared prediction error
Landsat.fit$Rmspe

## Fitted model parameters
Landsat.fit$margins
Landsat.fit$fit.par


