##############################################################################
## Simulation Function for Monte-Carlo Study
## When Sample size is different
##############################################################################

SA.simMC = function(n, N, X.mat, family, tau, beta, mar.dist, mar.par, areaID, B){
  
  # Obtain Copula Parameter:
  
  if(family=="normal"){famnum = 1}
  if(family=="t"){famnum = 2}     # Caution: be careful with the second parameter of t copula.
  if(family=="clayton"){famnum = 3}
  if(family=="gumbel"){famnum = 4}
  if(family=="frank"){famnum = 5}
  
  par = VineCopula::BiCopTau2Par(family=famnum, tau=tau)
  
  
  # Generate Population:
  
  cop.data = SA.rCopula(N=N, family=family, par=par)
  
  U = cop.data$U
  AreaID = cop.data$AreaID  
  Error = SA.U2Error(U, mar.dist = mar.dist, mar.par=mar.par)
  
  Pop.data <- SA.rPop(N=N, AreaID=AreaID, Error=Error, X.mat=X.mat, beta = beta)
  Y.mat <- Pop.data$Y.mat                         
  
  Y.bar <- Pop.data$Y.bar 
  X.bar <- apply(X.mat,1,mean)
  
  
  # Obtain Sample:
  
  Sample.data <- SA.rSample(n=n, Y.mat, X.mat)
  y <- Sample.data$y                        
  x <- Sample.data$x 
  error <-  y - (beta[1] + x*beta[2])    # true error
  u <- SA.Error2U(error, mar.dist = mar.dist, mar.par=mar.par)
  
  #areaID <- rep(1:m, each=n[1])  # Caution: be careful if sample sizes are different.
  
  
  
  # Benchmark: BUP under the true model:
  
  e.hat.true <- SA.pred.error(error=error, u=u, areaID=areaID, family=family, par=par)  # true copula 
  e.bar.true <- aggregate(error, by = list(areaID), FUN = "mean")$x 
  Yi_BUP <-  SA.pred.mean(N=N, n=n, e.hat = e.hat.true, e.bar=e.bar.true, beta = beta, X.bar=X.bar) # true beta
  
  
  
  # Approach 1: Parametric Estimation
  
  results.par <- SA.Est.par(y=y, x=x, areaID=areaID, family=family, mar.dist=mar.dist) 
  e.hat.par   <- SA.pred.error(error=results.par$resids, u=results.par$u, areaID=areaID, 
                               family=family, par=results.par$par) 
  Yi_EBUP.par <-SA.pred.mean(N=N, n=n, e.hat = e.hat.par, e.bar=results.par$residbar, 
                             beta = results.par$beta, X.bar=X.bar)
  
  
  
  # Approach 2: Semi-parametric Estimation
  
  results.spar <- SA.Est.spar(y=y, x=x, areaID=areaID, family=family, est.method= "mle") 
  e.hat.spar <- SA.pred.error(error=results.spar$resids, u=results.spar$u, areaID=areaID, 
                              family=family, par=results.spar$par) 
  Yi_EBUP.spar <-SA.pred.mean(N=N, n=n, e.hat = e.hat.spar, e.bar=results.spar$residbar, 
                              beta = results.spar$beta, X.bar=X.bar)
  
  
  
  ## 1. Fully parametric approach 
  ##  bias and mse of EBUP -- averaged over m small areas
  Bias.par <- ( Yi_EBUP.par - Y.bar)
  Bias.par <- aggregate(Bias.par,list(n),mean)$x
  
  
  MSE.par  <- ((Yi_EBUP.par - Y.bar)^2)
  MSE.par <- aggregate(MSE.par, list(n), mean)$x
  
  MSE1.par <- ((Yi_BUP      - Y.bar)^2)
  MSE1.par <- aggregate(MSE1.par, list(n), mean)$x
  
  
  MSE2.par <- ((Yi_EBUP.par - Yi_BUP )^2)
  MSE2.par <- aggregate(MSE2.par, list(n), mean)$x
  
  MSE3.par <- (2*(Yi_BUP - Y.bar)*(Yi_EBUP.par - Yi_BUP))
  MSE3.par <- aggregate(MSE3.par, list(n), mean)$x
  
  
  
  ## 2.Semi-parametric approach
  ##  bias and mse of EBUP -- averaged over m small areas
  Bias.spar <- ( Yi_EBUP.spar - Y.bar)
  Bias.spar <- aggregate(Bias.spar,list(n),mean)$x
  
  
  MSE.spar  <- ((Yi_EBUP.spar - Y.bar)^2)
  MSE.spar <- aggregate(MSE.spar, list(n), mean)$x
  
  MSE1.spar <- ((Yi_BUP      - Y.bar)^2)
  MSE1.spar <- aggregate(MSE1.spar, list(n), mean)$x
  
  
  MSE2.spar <- ((Yi_EBUP.spar - Yi_BUP )^2)
  MSE2.spar <- aggregate(MSE2.spar, list(n), mean)$x
  
  MSE3.spar <- (2*(Yi_BUP - Y.bar)*(Yi_EBUP.spar - Yi_BUP))
  MSE3.spar <- aggregate(MSE3.spar, list(n), mean)$x
  
  
  
 
  
  
  ### BOOTSTRAP for MSE estimation:
  boot.1.par <- boot.1.spar <- matrix(NA, ncol = 3)

  for(bb in 1:B){
    
    boot.par <- SA.simBoot.par(n=n, N=N, X.mat = X.mat, family=family, par= results.par$par, 
                                    beta= results.par$beta, mar.dist=mar.dist, mar.par=results.par$mar.par, areaID)
    
    boot.spar <- SA.simBoot.spar(n=n, N=N, X.mat = X.mat, family=family, par= results.spar$par, 
                                      beta= results.spar$beta, obs.error = results.spar$resids, 
                                      est.method="mle", areaID)
    
    boot.2.par <- boot.par
    boot.1.par <- rbind(boot.1.par, boot.2.par)
    
    boot.2.spar <- boot.spar
    boot.1.spar <- rbind(boot.1.spar, boot.2.spar)
    
    
  }
  
  boot.1.par <- boot.1.par[-1,]
  boot.1.spar <- boot.1.spar[-1,]
  
  mean.boot.1.par.bias <- aggregate(boot.1.par[,2], list(boot.1.par[,1]), mean)$x
  mean.boot.1.par.mse <- aggregate(boot.1.par[,3], list(boot.1.par[,1]), mean)$x
  
  mean.boot.1.spar.bias <- aggregate(boot.1.spar[,2], list(boot.1.spar[,1]), mean)$x
  mean.boot.1.spar.mse <- aggregate(boot.1.spar[,3], list(boot.1.spar[,1]), mean)$x
  
  
  
  par.hat  = cbind(results.par$par, results.spar$par)
  tau.hat  = VineCopula::BiCopPar2Tau(family=famnum, par = par.hat)
  beta.hat =  results.par$beta       
  mpar.hat = results.par$mar.par
  
  
  Bias = cbind(Bias.par, Bias.spar)
  MSE = cbind(MSE.par, MSE.spar)
  MSE1 = cbind(MSE1.par, MSE1.spar)
  MSE2 = cbind(MSE2.par, MSE2.spar)
  MSE3 = cbind(MSE3.par, MSE3.spar)
  
  Bias.boot = cbind(mean.boot.1.par.bias, mean.boot.1.spar.bias)
  MSE.boot = cbind(mean.boot.1.par.mse, mean.boot.1.spar.mse)
  
  result = cbind(group = unique(n), par.par = par.hat[,1], par.spar= par.hat[,2], tau.par = tau.hat[,1],tau.spar = tau.hat[,2], beta1 = beta.hat[1], beta2= beta.hat[2], mar.par1 = mpar.hat[1],mpar.hat2= mpar.hat[2], 
             Bias = Bias, MSE=MSE, MSE1= MSE1, MSE2=MSE2, MSE3= MSE3, 
             Bias.boot=Bias.boot, MSE.boot=MSE.boot)
  return(result)
  
  
  
}



