########################################################
# Small Area Estimation Functions:
########################################################
# Version: July 15, 2019


##############################################################################
# Simulation of small area unifom (copula) data
##############################################################################

SA.rCopula <- function(N, family, par){
  
  cop.data <- NULL
  
  for(i in 1:length(N)){
    
    if(N[i] == 1) {  ## If dim=1, it is calculated using uniform distribution[0,1]
      
      U <- runif(1)
      
    }else {          ## If dim >=2, it is calculated using Clayton Copula
      
      if (family == c("normal")){ ## Gaussian Copula
        
        mycop <- copula::normalCopula(par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
      
      if (family == c("t")){ ## t Copula
        
        mycop <- copula::tCopula(par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
      
      if(family == "clayton" | family == "gumbel" | family == "frank" | family == "joe") { 
        
        mycop <- copula::archmCopula(family = family, par, dim = N[i])
        U <- t(copula::rCopula(1, mycop))
      }
    }
    
    AreaID <- as.matrix(seq(from = i, to = i, length.out = N[i])) ## Constructing a new variable which displays the areaID, thus to distinguish between each area
    U.area <- cbind(U, AreaID)
    cop.data <- rbind(cop.data, U.area)
  }
  
  return(list(U = cop.data[,1], AreaID = cop.data[,2]))
  
}



######################################################
# example: 20 small areas each with size 200.
#####################################################
# m = 20 ; N.i = 200
# N = rep(N.i,m)
# cop.data = SA.rCopula(N=N, family="clayton", par=2)
# U = cop.data$U
# AreaID = cop.data$AreaID  # (note: capital A for population, small a for sample)
######################################################


##############################################################################
# Converting Copula Data to Error using a Parametric Model
##############################################################################

SA.U2Error <- function(u, mar.dist, mar.par){
  
  if(mar.dist == "norm"){
    if(length(mar.par)!=2) stop("Provide mean and variance")
    error <- qnorm(u, mean = mar.par[1], sd = sqrt(mar.par[2])) ## Calculating Inverse CDF
  }
  
  
  if(mar.dist == "snorm"){
    if(length(mar.par)!=3) stop("Provide three parameters")
    error <- qsn(u, xi=mar.par[3], omega = mar.par[1], alpha = mar.par[2]) ## Calculating Inverse CDF
  }
  
  return(error) 
}




# example (cont'd) 
############################################################
# Error = SA.U2Error(U, mar.dist = "norm", mar.par=c(0,1))




##############################################################################
# Converting Error to Copula Data using a Parametric Model
##############################################################################

SA.Error2U <- function(error, mar.dist, mar.par){
  
  if(mar.dist == "norm"){
    if(length(mar.par)!=2) stop("Provide mean and variance")
    u <- pnorm(error, mean = mar.par[1], sd = sqrt(mar.par[2])) ## Calculating CDF
  }
  
  if(mar.dist == "snorm"){
    if(length(mar.par)!=3) stop("Provide three parameters")
    u <- psn(error, xi=mar.par[3], omega = mar.par[1], alpha = mar.par[2]) ## Calculating CDF
  }
  
  return(u) 
}







##############################################################################
# Simulation of small area population data
##############################################################################

SA.rPop <- function(N, AreaID, Error, X.mat, beta){
  
  m <- length(unique(AreaID))
  if(length(N) != m) stop("Provide an m-dimensional vector of population size.")
  get.e <- cbind(Error, AreaID)
  error.mat <- matrix(get.e[,1], nrow = m, ncol = N, byrow = TRUE)
  
  # Generate population:
  Y.mat <- matrix(NA, nrow=m, ncol=N)  ## Define Y matrix to restore values
  
  for(i in 1:m){
    for(j in 1:N[i]){
      Y.mat[i,j]<- beta[1]+beta[2]*X.mat[i,j] + error.mat[i,j] 
    }
  }
  
  Y.bar <- apply(Y.mat,1,mean)  # small area means for response
  
  return(list(Y.mat = Y.mat, Y.bar = Y.bar))
}


# example (cont'd) 
############################################################
# draw X from  X from N(1,1)
# set.seed(123000)
# X.mat <- matrix(c(rnorm(N.i*m, mean = 1, sd = 1)), m, N.i) 
# X.bar <- apply(X.mat, 1, mean)
# Pop.data <- SA.rPop(N=rep(N.i,m), AreaID, Error, X.mat, beta = c(1,1))
# Y.mat <- Pop.data$Y.mat                         
# Y.bar <- Pop.data$Y.bar 





##############################################################################
# Select Sample  without replacement from populations of Y and X:
##############################################################################

SA.rSample <- function(n, Y.mat, X.mat){
  
  m <- nrow(X.mat)
  N.i <- ncol(X.mat)
  
  if(nrow(Y.mat) != m | ncol(Y.mat) != N.i) stop("Population dimensions for X and Y do not match.")
  
  if(length(n) != m) stop("Provide an m-dimensional vector of sample size.")
  
  x <- y <- rep(NA, m*n[1])  ## define x, y and usamp vector to restore values
  N <- rep(m,N.i)
  a<-1
  b<-n[1]
  
  for(i in 1:m){
    no.ran <- sample(1:N[i], n[i], replace=FALSE, prob=NULL)
    y[a:b] <- Y.mat[i,no.ran]
    x[a:b] <- X.mat[i,no.ran]
    a <- b+1
    b <- a+n[1]-1
  }
  
  return(list(y=y, x=x))
}


# example (cont'd) 
############################################################
# n.i = 4
# n=rep(n.i,m)
# Sample.data <- SA.rSample(n=n, Y.mat, X.mat)
# y <- Sample.data$y                        
# x <- Sample.data$x 
## epsilon <-  y - (beta[1] + x*beta[2])   # true error from sampled observation




##############################################################################
# Copula log-likehood function for estimation of the copula parameter
##############################################################################
SA.LLIK <- function(u, areaID, family, par){
  
  n.area = as.numeric(table(areaID))
  cop.dens = rep(NA,length(n.area))
  
  for(i in 1:length(n.area)){
    
    if(n.area[i]>1){
      u.area <- subset(u, areaID == i)
      
      if(family == "normal"){
        mycop <- copula::normalCopula(par=par , dim= n.area[i])
      }
      
      if(family == "t"){
        mycop <- copula::tCopula(par=par , dim= n.area[i])
      }
      
      if(family == "clayton" | family == "gumbel" | family == "frank") { 
        mycop <- copula::archmCopula(family = family, par=par , dim= n.area[i])
      }
      
      cop.dens[i] <- copula::dCopula(u.area, mycop, log = TRUE)
    }
    else{cop.dens[i] <- 0}
  }
  
  return(sum(cop.dens[is.finite(cop.dens)], rm.na=T))
}





##############################################################################
# Likelihood-based Estimation of the Copula Parameter
##############################################################################

SA.CopEst <- function(u, areaID, family){
  
  obj.function= function(par){-SA.LLIK(u, areaID, family=family, par=par)}
  
  if(family == "normal" | family == "t"){
    res<-optimize(obj.function, lower=0, upper=1)
  }
  
  if(family == "gumbel"){
    res<-optimize(obj.function, lower=1, upper=100)
  }
  
  if(family == "frank" | family == "clayton"){
    res<-optimize(obj.function, lower=0, upper=10)
  }
  
  return(list(par = res$minimum, llik = -res$objective ))
}





#############################################################################
## Weight Function (w1i)
##############################################################################

SA.weight <- function(u,  u.i,  family, par){
  
  n <- length(u)       ## CDF of all residuals
  nSAi <- length(u.i)  ## CDF of selected small area
  
  Fe_matrix <- matrix(u.i, nrow = n, ncol = nSAi, byrow = TRUE)
  
  if(nSAi != 1){
    if(family == "normal"){
      cop.num <- copula::normalCopula( par, dim = (nSAi+1))
      cop.den <- copula::normalCopula( par, dim = nSAi)
    }
    
    if(family == "t"){
      cop.num <- copula::tCopula( par, dim = (nSAi+1))
      cop.den <- copula::tCopula( par, dim = nSAi)
    }
    
    if(family == "clayton" | family == "gumbel" | family == "frank") { 
      cop.num <- copula::archmCopula(family = family, par, dim = (nSAi+1))
      cop.den <- copula::archmCopula(family = family, par, dim = nSAi)
    }
    
    w1.i_num <- copula::dCopula(cbind(u, Fe_matrix), cop.num)
    w1.i_den <- copula::dCopula(u.i, cop.den)
    w1.i <- w1.i_num/w1.i_den
  }
  
  else {
    if(family == "normal"){
      cop.num <- copula::normalCopula( par, dim = (nSAi+1))
    }
    
    if(family == "t"){
      cop.num <- copula::tCopula( par, dim = (nSAi+1))
    }
    
    if(family == "clayton" | family == "gumbel" | family == "frank") { 
      cop.num <- copula::archmCopula(family = family, par, dim = (nSAi+1))
    }
    
    w1.i_num <- copula::dCopula(cbind(u, Fe_matrix), cop.num)
    w1.i <- w1.i_num
  }
  
  return(w1.i)
}



#############################################################################
## Empirical predictor function for unsampled error terms
##############################################################################

SA.pred.error <- function(error, u, areaID, family, par){
  
  m <- length(unique(areaID))
  
  emp.pred.areai <- NULL     
  for(i in 1:m){
    u.i <- subset(u, areaID == i)
    w1.i <- SA.weight(u=u, u.i= u.i,  family = family, par=par)
    emp.pred.areai[i] <- sum(error * w1.i) / sum(w1.i)
  }
  return(emp.pred.areai)
}







#############################################################################
## Empirical BUP / BUP (Depending upon values)
##############################################################################

SA.pred.mean <- function(N, n, e.hat, e.bar, beta, X.bar){
  
  m <- length(X.bar)
  if(length(N) != m | length(n) != m) Stop("Number of small areas does not match for sample and/pr population.")
  
  Yi <- NULL
  
  for(i in 1:m){
    l1 <- (beta[1]+(X.bar[i]*beta[2]))
    l2 <- ((n[i]/N[i])*e.bar[i])
    l3 <- ((N[i]-n[i])/N[i])
    l4 <- e.hat[i]
    Yi[i] <-  l1 + l2 + (l3*l4)
  }
  
  return(Yi)
}




##############################################################################
# Small Area Estimation using a fully parametric model
##############################################################################


SA.Est.par = function(y, x, areaID, family, mar.dist, mar.par, beta){
  
  # Fitting the marginal models using LM package if beta is missing
  
  if(missing(beta)){
    fit.margin <- lm(y ~ x) 
    beta <- as.numeric(fit.margin$coefficients)
    resids <- as.numeric(resid(fit.margin))
  }else{
    resids <-  y - (beta[1] + x*beta[2])   # gives true error if true beta is given 
  }
  
  residbar <- aggregate(resids, by = list(areaID), FUN = "mean")$x  # Mean of residuals for each area
  
  # If missing, estimate mar.par under mar.dist:  
  
  if(missing(mar.par)){
    if(mar.dist == "norm"){ 
      mar.par <- c(mean(resids), var(resids))
    }
    if(mar.dist == "snorm"){ 
      x.sn <-  matrix(1, nrow = length(resids), ncol = 1)
      sn.est <-  sn::selm.fit(x.sn, resids, family = "SN", selm.control = "MLE")
      mar.par <- as.numeric(sn.est$param$dp[c(2,3,1)])
    }}
  
  # CDF transformation of residuals under mar.dist:
  
  u <- SA.Error2U(error = resids, mar.dist=mar.dist, mar.par=mar.par)
  cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
  par = cop.est$par
  
  return(list(resids = resids, residbar = residbar,  u = u, par = par, beta=beta,  mar.par=mar.par))
}




##############################################################################
# Small Area Estimation using a semi-parametric model 
##############################################################################

SA.Est.spar = function(y, x, areaID, family, est.method= c("mle"), beta){
  
  # Fitting the marginal models using LM package if beta is missing
  
  if(missing(beta)){
    fit.margin <- lm(y ~ x) 
    beta <- as.numeric(fit.margin$coefficients)
    resids <- as.numeric(resid(fit.margin))
  }else{
    resids <-  y - (beta[1] + x*beta[2])   # gives true error if true beta is given 
  }
  
  residbar <- aggregate(resids, by = list(areaID), FUN = "mean")$x  # Mean of residuals for each area
  
  # CDF transformation of residuals under unknown marginal distribution
  
  u <- copula::pobs(resids)
  if(est.method =="mle"){
    cop.est <- SA.CopEst(u=u, areaID=areaID, family=family)
    par = cop.est$par
  }
  
  return(list(resids = resids, residbar = residbar,  u = u, par = par, beta=beta))
}






##################################################################################
### Parametric Bootstrap Function 
##################################################################################


SA.simBoot.par <- function(n, N, X.mat, family, par, beta, mar.dist, mar.par){
  
  # Generate Population:
  
  cop.data = SA.rCopula(N=N, family=family, par=par)
  
  U = cop.data$U
  AreaID = cop.data$AreaID  
  Error = SA.U2Error(U, mar.dist = mar.dist, mar.par=mar.par)
  
  Pop.data <- SA.rPop(N=N, AreaID=AreaID, Error=Error, X.mat=X.mat, beta = beta)
  Y.mat <- Pop.data$Y.mat                         
  
  Y.bar <- Pop.data$Y.bar 
  X.bar <- apply(X.mat,1,mean)
  
  
  # Obtain Sample:
  
  Sample.data <- SA.rSample(n=n, Y.mat, X.mat)
  y <- Sample.data$y                        
  x <- Sample.data$x 
  error <-  y - (beta[1] + x*beta[2])    
  areaID <- rep(1:m, each=n[1])  # Caution: be careful if sample sizes are different.
  
  # Parametric Estimation
  
  results.par <- SA.Est.par(y=y, x=x, areaID=areaID, family=family, mar.dist=mar.dist) 
  e.hat.par   <- SA.pred.error(error=results.par$resids, u=results.par$u, areaID=areaID, 
                               family=family, par=results.par$par) 
  Yi_EBUP.par <-SA.pred.mean(N=N, n=n, e.hat = e.hat.par, e.bar=results.par$residbar, 
                             beta = results.par$beta, X.bar=X.bar)
  
  
  ## MSE estimation calculation
  Bias.boot.par <- (Yi_EBUP.par - Y.bar)
  MSE.boot.par <- ((Yi_EBUP.par - Y.bar)^2)
  
  boot.result <- c(unique(areaID), Bias.boot.par, MSE.boot.par)
  boot.result <- matrix(boot.result, nrow = m, ncol=3);
  return(boot.result)
}





##################################################################################
### Semi-Parametric Bootstrap Function 
##################################################################################


SA.simBoot.spar <- function(n, N, X.mat, family, par, beta, obs.error, est.method=c("mle", "eTau")){ 
  
  # Generate Population:
  
  cop.data = SA.rCopula(N=N, family=family, par=par)
  
  U = cop.data$U
  AreaID = cop.data$AreaID  
  
  # Quantile mapping in the range of observed error (residuals) based on sample:
  
  Error = rep(NA,length(U))
  sort.e = sort(obs.error)
  sort.v = copula::pobs(sort.e)
  
  for(j in 1:length(U)){
    if(U[j]>=max(sort.v)){ Error[j]=max(obs.error)
    }else if( U[j] <= min(sort.v)){ Error[j]=min(obs.error)
    }else{
      loc = findInterval(U[j], sort.v)
      v.int = sort.v[c(loc, loc+1)] 
      e.int = sort.e[c(loc, loc+1)] 
      Error[j] = approx(v.int, e.int, xout= U[j], yleft= min(obs.error), yright=max(obs.error))$y
    }}
  
  
  Pop.data <- SA.rPop(N=N, AreaID=AreaID, Error=Error, X.mat=X.mat, beta = beta)
  Y.mat <- Pop.data$Y.mat                         
  
  Y.bar <- Pop.data$Y.bar 
  X.bar <- apply(X.mat,1,mean)
  
  # Obtain Sample:
  
  Sample.data <- SA.rSample(n=n, Y.mat, X.mat)
  y <- Sample.data$y                        
  x <- Sample.data$x 
  error <-  y - (beta[1] + x*beta[2])    
  areaID <- rep(1:m, each=n[1])  # Caution: be careful if sample sizes are different.
  
  
  
  # Semi-parametric Estimation
  
  results.spar <- SA.Est.spar(y=y, x=x, areaID=areaID, family=family, est.method = est.method) 
  e.hat.spar <- SA.pred.error(error=results.spar$resids, u=results.spar$u, areaID=areaID, 
                              family=family, par=results.spar$par) 
  Yi_EBUP.spar <-SA.pred.mean(N=N, n=n, e.hat = e.hat.spar, e.bar=results.spar$residbar, 
                              beta = results.spar$beta, X.bar=X.bar)
  
  
  ## MSE estimation calculation
  Bias.boot.spar <- (Yi_EBUP.spar - Y.bar)
  MSE.boot.spar <- ((Yi_EBUP.spar - Y.bar)^2)
  
  boot.result <- c(unique(areaID),Bias.boot.spar, MSE.boot.spar)
  boot.result <- matrix(boot.result, nrow = m, ncol=3)
  return(boot.result)
}



